
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;            Autocad Calendar copied from Cadence magazine.
;
;
;                       Ronald A. Kalister
;                        Brunswick, Ohio.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun C:CALENDAR( / mn dy cd cel ar crx cry qu
                      d c y m ox oy xcl ycl am nd cc loc m0 y0)
;Initial settings and definition of constants
  (setvar "CMDECHO" 0)
  (setvar "BLIPMODE" 0)
  (setq mn '("JANUARY" "FEBRUARY" "MARCH" "APRIL" "MAY" "JUNE" "JULY"
             "AUGUST" "SEPTEMBER" "OCTOBER" "NOVEMBER" "DECEMBER")
        dy '(31 28 31 30 31 30 31 31 30 31 30 31)
        cd '(1 4 4 0 2 5 0 3 6 1 4 6))
  (setq cel 1.5                    ; Cell size
        ar 0.8333                  ;Aspect ratio [to fit into A-size sheet]
        crx 0.0                    ;Lower left-hand corner reference
        cry 0.0
        qu 1)                      ;Quadrant [0=centered, 1=lower right]
   (command ".Style" "TXA" "romant" "" "0.75" "" "" "" "")  ;For characters
   (command ".Style" "TXB" "romans" "" "0.75" "" "" "" "")  ;For numbers
   (setq d (rtos (getvar "CDATE") 2 0)                 ;Get system date
         c (atoi (substr d 1 2))                       ;Century
         y (atoi (substr d 3 2))                       ;Year, and
         m (atoi (substr d 5 2))                       ;Month.
         xcl cel                                       ;Cell size - X
         ycl (* ar xcl)                                ;Cell size - Y
         am (strcat (nth (1- m) mn) " " (substr d 1 4)) ;Set month string,
         nd (nth (1- m) dy)                            ;Number of days, and
         cc (+ (nth (1- m) cd) (- 19 c)))              ; Month code.
    (princ " Generating calendar for ") (princ am)
    (princ ", please wait . .")
;CALDRAW returns the day of the week and location of the last date.
  (setq loc (caldraw qu 0 crx cry xcl ycl m y am nd cc))
;To generate mini calendars for the previous and next months:
  (if (or (zerop (car loc)) (> (car loc) 2))
     (if (< (cadr loc) (+ crx (* xcl 5.0)))
       (setq ox (+ crx (* xcl 5.0))
             oy (+ cry (* ycl 0.05)))
       (setq ox crx
             oy (+ cry (* ycl 4.05))))
      (if (< (caddr loc) (+ cry ycl))
        (setq ox (+ crx (* xcl 5.0))
              oy (+ cry (* ycl 0.05)))
        (setq ox (+ crx (* xcl (+ (car loc) 1)))
              oy (+ cry (* ycl 4.05)))))
        (setq xcl (/ xcl 7.0)                  ;Cell size - X
              ycl (* ar xcl))                  ;Cell size - Y
;For the previous month:
  (if (= m 1)                                  ;Wrap around if January
    (setq m0 12 y0 (1- y))
    (setq m0 (1- m) y0 y))
  (setq am (nth (1- m0) mn)                    ;Set month string
        nd (nth (1- m0) dy)                    ;Number of days, and
        cc (+ (- 19 c) (nth (1- m0) cd)))      ;Month code.
  (if (minusp y0) (setq y0 99 cc (1+ cc)))    ;Change of century
  (CALDRAW 0 1 ox oy xcl ycl m0 y0 am nd cc)
;For the next month:
  (setq	ox (+ ox (* xcl 7.0)))
  (if (= m 12)                                  ;Wrap around if December
     (setq m0 1 y0 (1+ y))
     (setq m0 (1+ m) y0 y))
   (setq am (nth (1- m0) mn)                     ;Set month string,
         nd (nth (1- m0) dy)                    ;Number of days, and
         cc (+ (nth (1- m0) cd) (- 19 c)))      ;Month code.
   (if (> y0 99) (setq y0 0 cc (1- cc)))        ;Change of century
   (CALDRAW 0 1 ox oy xcl ycl m0 y0 am nd cc)
   (command ".ZOOM" "E")
   (princ " . complete.\n  Save the drawing and use PRPLOT or PLOT.\n")
   (princ "\nCALENDAR -- from A's Computing Expertise - (609) 772-1309\n")
   (princ)
)
;This function actually generates the calendar.
(defun CALDRAW (qflg dflg xo yo xcl ycl mn yr am nd cc
                / WK dw cfx cfy ta re te x y ht i)
   (setq WK ' ("SUN" "MON" "TUE" "WED" "THU" "FRI" "SAT"))
   (if (and (= mn 2) (zerop (rem yr 4))) (setq nd 29)) ;Leap year corrections
   (if (and (zerop (rem yr 4)) (or (= mn 1) (= mn 2)))
          (setq cc (1- cc)))
;Compute the day of the week for the 1st; 1=Sun, 0,7=Sat.
    (setq dw (rem (+ yr (/ yr 4) 1 cc) 7))
    (if (zerop dw) (setq dw 7))
    (if (zerop qflg)
        (setq cfx 0.0 cfy 0.0 ta "M")
        (setq cfx (* 0.45 xcl) cfy (* 0.45 ycl) ta "R"))
    (setq re (+ xo (* 7.0 xcl))              ;Define right edge and
          te (+ yo (* 5.5 ycl))              ;Top edge of frames.
          x (+ xo (* 3.5 xcl)))              ;To write month,
    (command ".Text" "S" "TXA") (command)    ;Reset text style.
    (command ".Text" "C" (list x (* te 1.01)) (* xcl 0.3333) "0" am)
    ;Draw the calendar frames.
    (if (zerop dflg) (progn                   ;Only for the main calendar
           (setq x xo y yo)
           (repeat 8                           ;Draw vertical lines
              (command ".Line" (list x yo) (list x te) "")
             (setq x (+ x xcl)))
           (repeat 6                            ;Draw horizontals
              (command ".Line" (list xo y) (list re y) "")
              (setq y (+ y ycl)))
           (command ".Line" (list xo te) (list re te) "")   ;Draw top edge
           (setq x (+ xo (* 0.5 xcl))            ;Set values for writing
                 y (- te (* 0.25 ycl))           ;The days of the week
                 ht (* ycl 0.25)
                 i 0)
           (repeat 7                               ;Write days
               (command ".Text" "M" (list x y) ht "0" (nth i WK))
               (setq x (+ x xcl)
                     i (1+ i))))); IF ZEROP DFLG
             (command ".Text" "S" "TXB") (command)     ;Set text style and
             (setq x (+ xo (* (- dw 1.5) xcl) cfx)     ;Starting point - X
                   y (- (+ yo (* 4.5 ycl)) cfy)         ;Starting point - Y
                   ht (* ycl 0.5)                      ;Text height and
                   i 0)                                ;Date
              (repeat nd                               ;To write the dates
                (setq x (+ x xcl)
                      i (1+ i))
                (if (> x re)                            ;To go to next row
                   (setq x (+ xo (* 0.5 xcl) cfx)
                         y (- y ycl)))
                 (if (< y yo)                            ;To go to top row
                    (setq y (- (+ yo (* 4.5 ycl)) cfy)))
                (command ".Text" ta (list x y) ht "0" (itoa i)))          
   ;Return the day of the week and the location of the last date.
                (setq dw (rem (+ (1- dw) nd) 7))
                (list dw x y))
   ;Execute the program automatically upon loading.
   (CALENDAR)              


                            
